/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "FrameData.hh"
#include "DVector.hh"
#include <stdexcept>

using namespace std;

namespace generator {

  FrameData::InpList  FrameData::mInputList;

  //====================================  Frame data source constructor
  FrameData::FrameData(void) {
      setSample(Interval(0.0));
      mInput = mInputList.end();
  }

  //====================================  Frame data source constructor
  FrameData::FrameData(const char* files, const char* channel) {
      setSample(Interval(0.0));
      mInput = mInputList.end();
      if (files)   setFileList(files);
      if (channel) setChannel(channel);
  }

  //====================================  Frame data source destructor
  FrameData::~FrameData(void) {
  }

  //====================================  Clone a frame data source
  FrameData* 
  FrameData::clone(void) const {
      return new FrameData(*this);
  }

  //====================================  Get the default sample-rate;
  double
  FrameData::defaultRate(const Time& t0, Interval dT) {
      double rate = 0;
      if (mInput != mInputList.end()) {
	  mInput->readDataStride(t0, dT);
	  TSeries* ts = mInput->refDacc().refData(mChannel.c_str());
	  rate = double(ts->getTStep());
	  if (rate != 0) rate = 1.0/rate;
      }
      return rate;
  }

  //====================================  Get the default data type
  long
  FrameData::defaultType(const Time& t0, Interval dT) {
      long dty = -1;
      if (mInput != mInputList.end()) {
	  mInput->readDataStride(t0, dT);
	  TSeries* ts = mInput->refDacc().refData(mChannel.c_str());
	  dty = ts->refDVect()->getType();
      }
      return dty;
  }

  //====================================  Set the channel ID
  void 
  FrameData::setChannel(const string& value) {
      mChannel = value;
      if (mInput != mInputList.end()) {
	  mInput->refDacc().addChannel(mChannel.c_str());
      }
      DataSource::setParameter("Channel", mChannel);
  }

  //====================================  Set the file list
  void 
  FrameData::setFileList(const string& value) {
      mInput = findInput(value);
      if (mInput == mInputList.end()) mInput = addInput(value);
      mInput->refDacc().setDebug(getDebug());
      if (!mChannel.empty()) {
	  mInput->refDacc().addChannel(mChannel.c_str());
      }
      DataSource::setParameter("Files", value);
  }

  //====================================  Set an arbitrary parameter
  void 
  FrameData::setParameter(const string& name, const string& value) {
      if (getDebug()) cout << "Setting FrameData::" << name << " to " 
			   << value << endl;
      if (name == "Channel")    setChannel(value);
      else if (name == "Files") setFileList(value);
      else                      DataSource::setParameter(name, value);
  }

  FrameData::input::input(const char* fl) {
      if (fl) setFileList(fl);
  }

  FrameData::input::~input(void) {
  }

  void 
  FrameData::input::setFileList(const string& fList) {
      mFileList = fList;
      mIn.addFile(fList.c_str());
  }

  void 
  FrameData::input::readDataStride(const Time& t0, Interval dT) {
      if (mIn.getFillTime() != Time(0) && mIn.getFillTime() <= t0 && 
	  mIn.getCurrentTime() >= t0+dT) return;
      if (mIn.getCurrentTime() < t0) {
	  if (mIn.seek(t0)) throw runtime_error("Data not available");
      }

      if (mIn.getFillTime() > t0) throw runtime_error("Data not available");

      if (mIn.getCurrentTime() == t0) {
	  int rc = mIn.fillData(dT);
	  if (rc) {
	      cout << "Missing data in source " << mFileList << " at " 
		   << t0 << endl;
	      throw runtime_error("Data not available");
	  }
      }
 }

  void 
  FrameData::getSourceTSeries(const Time& t0, Interval dT, int N, 
			      TSeries& data) 
  {
      if (mInput != mInputList.end()) {
	  mInput->refDacc().setDebug(getDebug());
	  if (getDebug()) {
	    cout << "FrameData: getting channel: " << mChannel 
		 << " data for gps " << t0 << endl;
	  }
	  Interval duration = dT * double(N);
	  mInput->readDataStride(t0, duration);
	  TSeries* ts = mInput->refDacc().refData(mChannel.c_str());
	  data = ts->extract(t0, duration);
      }
  }

  FrameData::inp_iter 
  FrameData::findInput(const std::string& filelist) {
      for (inp_iter i=mInputList.begin() ; i != mInputList.end() ; ++i) {
	  if (filelist == i->getFileList()) return i;
      }
      return mInputList.end();
  }

  FrameData::inp_iter
  FrameData::addInput(const std::string& fileList) {
      mInputList.push_back(fileList.c_str());
      return --mInputList.end();
  }

}
